/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021-2022, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef headerfile_picotools_utf8_utf8_stream
#define headerfile_picotools_utf8_utf8_stream

#include "textstream/textstream.h"

namespace pt
{


/*!
	this function converts one UTF-8 character into one wide-character

	input:
		iterator_in - an TextStream iterator for reading from
		iterator_end - an end iterator (can be returned by end() method from TextStream)

	output:
		res - an output character
		correct - true if it is a correct character

		the function returns how many characters have been used from the input stream
*/
template<typename StreamIteratorType>
size_t utf8_to_int(
		StreamIteratorType & iterator_in,
		const StreamIteratorType & iterator_end,
		int & res,
		bool & correct)
{
size_t i, len;
unsigned char uz;

	res = 0;
	correct = false;

	if( iterator_in == iterator_end )
		return 0;

	uz = *iterator_in;
	++iterator_in;

	if( !private_namespace::utf8_to_int_first_octet(uz, len, res) )
		return 1;

	for(i=1 ; i<len ; ++i)
	{
		if( iterator_in == iterator_end )
			return i;

		uz = *iterator_in;
		++iterator_in;

		if( !private_namespace::utf8_to_int_add_next_octet(uz, res) )
			return i;
	}

	if( utf8_check_range(res, len) )
		correct = true;

return len;
}



}

#endif
